from django.db import models
from io import BytesIO
from PIL import Image
from django.core.files import File
from smartfields import fields
from django.db.models.signals import pre_delete, post_save
from django.dispatch.dispatcher import receiver
from django.conf import settings
import os
from django.utils import timezone

# image compression method
def compress(image):
    im = Image.open(image)
    im_io = BytesIO()
    im.save(im_io, 'JPEG', quality=40)
    new_image = File(im_io, name=image.name)
    return new_image


def get_name(instance, filename):
    return "{}/{}".format(instance.id, filename)


class Category(models.Model):
    cat_title = models.CharField(max_length=100, db_index=True)
    cat_slug = models.SlugField(max_length=150, unique=True, db_index=True)
    cat_description = models.TextField()
    cat_image = models.ImageField()
    is_active = models.BooleanField(default=True)

    def __str__(self):
        return self.cat_title

    def get_absolute_url(self):
        return reverse("shop:category", kwargs={'slug': self.slug })


class SubCategory(models.Model):
    category = models.ForeignKey(Category, related_name='subcategories', on_delete=models.CASCADE)
    sub_title = models.CharField(max_length=150, db_index=True)
    sub_slug = models.SlugField(max_length=150, unique=True, db_index=True)
    sub_description = models.TextField()
    sub_image = models.ImageField()
    is_active = models.BooleanField(default=True)

    def __str__(self):
        return str(self.category) +" - "+ str(self.sub_title)

    def get_absolute_url(self):
        return reverse("shop:subcategory", kwargs={'slug': self.slug })


# Create your models here.
class ProductData(models.Model):
    category = models.ForeignKey(Category, related_name='products', on_delete=models.CASCADE)
    subcategory = models.ForeignKey(SubCategory, related_name='products', on_delete=models.CASCADE)
    p_name = models.CharField(max_length=150, null=False, db_index=True)
    seller_name = models.CharField(max_length=70, null=True)
    s_username = models.CharField(max_length=70, null=True)
    p_img = fields.ImageField(upload_to='pics')
    p_img2 = fields.ImageField(upload_to='pics', null=True)
    p_img3 = fields.ImageField(upload_to='pics', null=True)
    p_img4 = fields.ImageField(upload_to='pics', null=True)
    p_price = models.IntegerField()
    min_q = models.CharField(max_length=10)
    p_desc = models.TextField(null=True)
    tag = models.CharField(max_length=20, null=True)
    created_at = models.DateTimeField(auto_now=True)

    def save(self, *args, **kwargs):
        x = self.p_img.name.split('.')
        if x[-1] == 'png' or x[-1] == 'PNG':
            png = Image.open(self.p_img)
            png.load()
            background = Image.new("RGB", png.size, (255, 255, 255))
            background.paste(png, mask=png.split()[3])
            new_im = BytesIO()
            background.save(new_im, 'JPEG', quality=80)
            new_image1 = File(new_im, name=self.p_img.name)
        else:
            new_image1 = self.p_img
        new_image = compress(new_image1)
        self.p_img = new_image
        super().save(*args, **kwargs)

    def get_absolute_url(self):
        return reverse('product_detail', args=(self.id,))

    @property
    def get_products(self):
        return Products.objects.filter(categories__title=self.title)


class UsersRated(models.Model):
    user = models.CharField(max_length=50, null=True)


class Rating(models.Model):
    users = models.ManyToManyField(UsersRated)
    seller = models.CharField(max_length=50, null=True)
    rate = models.IntegerField(default=0)
